import java.util.*;
import java.io.*;

import net.es.oscars.*;
import net.es.oscars.aaa.*;
import net.es.oscars.database.*;
import net.es.oscars.servlets.Utils;

import org.apache.log4j.*;
import org.hibernate.*;

/**
 * IDCCmdUtil an abstract class for creating command-line utilities
 *
 * @author Andrew Lake (alake@internet2.edu)
 */
public class IDCCmdUtil{
    protected String dbname;
    protected String aaaDbName;
    protected Logger log;
    
    /**
     * Method to read in user input strings
     *
     * @param in a Scanner used toaccept input
     * @param label a String describing the requested input to the user
     * @param defaultVal the default value to assign if no input provided
     * @param req boolean indicating whether this field is required
     * @return the String input by the user
     */
    protected String readInput(Scanner in, String label, String defaultVal, boolean req){
        System.out.print(label + (req?"*":""));// + " [" + defaultVal + "]: ");
        System.out.print(": ");
        String input = in.nextLine().trim();
        
        if(input.equals("") && (!defaultVal.equals(""))){
            input = defaultVal;
        }else if(input.equals("") && defaultVal.equals("") && req){
            System.err.println("The field '" + label + "' is required.");
            System.exit(0);
        }else if(input.equals("")){
            return null;
        }
        
        return input;
    }
    
    
    /**
     * Prints the current list of institutions in the database and allows the
     * user to choose one
     *
     * @param in the Scanner to use for accepting input
     * @return the selected Institution
     */
    protected Institution selectInstitution(Scanner in, String label){
        InstitutionDAO instDAO = new InstitutionDAO(this.dbname);
        List<Institution> institutions = instDAO.list();
        int i = 1;
        
        System.out.println();
        for(Institution inst : institutions){
            System.out.println(i + ". " + inst.getName());
            i++;
        }
        
        System.out.print("Select the " + label + " (by number): ");
        int n = in.nextInt();
        in.nextLine();
        
        if(n <= 0 || n > institutions.size()){
            System.err.println("Invalid organization number '" +n + "' entered");
            System.exit(0);
        }
        
        return institutions.get(n-1);
    }
}